#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
查询指定端口被哪些进程占用（跨平台）。

优先使用 psutil（若安装），否则基于系统自带工具：
- Windows: netstat + tasklist
- Linux: ss（优先）/netstat
- macOS: lsof
"""

from __future__ import annotations

import platform
import re
import shutil
import subprocess
from typing import List, Set, Tuple, Optional

# 用于按进程名查端口时复用“全量端口列表”
def _get_all_ports() -> List[Tuple[int, int, str, str, str]]:
    """返回 List[(port, pid, name, status, cmdline)]"""
    try:
        from .port_list import get_all_ports
        return get_all_ports()
    except ImportError:
        from port_list import get_all_ports
        return get_all_ports()


def _run_command(command_args: List[str]) -> Tuple[int, str]:
    """运行命令并返回 (returncode, stdout)"""
    process = subprocess.Popen(
        command_args,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        encoding='utf-8',
        errors='replace',
    )
    assert process.stdout is not None
    output_lines = []
    for line in process.stdout:
        output_lines.append(line)
    process.wait()
    return process.returncode, ''.join(output_lines)


def _try_psutil(port: int) -> List[Tuple[int, str]]:
    try:
        import psutil  # type: ignore
    except Exception:
        return []

    pid_set = set()
    results: List[Tuple[int, str]] = []
    try:
        for conn in psutil.net_connections(kind='inet'):
            laddr = conn.laddr if conn.laddr else None
            if not laddr:
                continue
            if getattr(laddr, 'port', None) != port:
                continue
            pid = conn.pid
            if pid is None or pid in pid_set:
                continue
            try:
                proc = psutil.Process(pid)
                name = proc.name()
            except Exception:
                name = "<unknown>"
            pid_set.add(pid)
            results.append((pid, name))
    except Exception:
        return []
    return results


def _windows_lookup(port: int) -> List[Tuple[int, str]]:
    rc, out = _run_command(["netstat", "-ano"])
    if rc != 0:
        return []
    pids = set()
    results: List[Tuple[int, str]] = []
    pattern = re.compile(r"^\s*(TCP|UDP)\s+[^\s]*:%d\b.*\s(\d+)\s*$" % port, re.IGNORECASE)
    for line in out.splitlines():
        m = pattern.match(line)
        if not m:
            continue
        pid = int(m.group(2))
        if pid in pids:
            continue
        pids.add(pid)
    for pid in sorted(pids):
        # 获取进程名
        rc2, out2 = _run_command(["tasklist", "/FI", f"PID eq {pid}", "/FO", "CSV", "/NH"])  # CSV 无表头
        name = "<unknown>"
        if rc2 == 0:
            # CSV 格式："Image Name","PID","Session Name","Session#","Mem Usage"
            # 取第一列作为进程名，去掉引号
            first_line = out2.strip().splitlines()[0] if out2.strip() else ""
            if first_line and first_line[0] == '"':
                try:
                    name = first_line.split('",')[0].strip('"')
                except Exception:
                    pass
        results.append((pid, name))
    return results


def _linux_lookup(port: int) -> List[Tuple[int, str]]:
    # 优先使用 ss
    if shutil.which("ss") is not None:
        rc, out_tcp = _run_command(["ss", "-ltnp"])  # TCP
        rc2, out_udp = _run_command(["ss", "-lunp"])  # UDP
        combined = (out_tcp if rc == 0 else "") + "\n" + (out_udp if rc2 == 0 else "")
        # 示例: users:(("nginx",pid=1234,fd=6))
        results: List[Tuple[int, str]] = []
        seen = set()
        for line in combined.splitlines():
            if f":{port} " not in line and not line.strip().endswith(f":{port}"):
                continue
            for name, pid in re.findall(r"users:\(\(\"([^\"]+)\",pid=(\d+)", line):
                pid_i = int(pid)
                if pid_i in seen:
                    continue
                seen.add(pid_i)
                results.append((pid_i, name))
        if results:
            return results
    # 回退使用 netstat
    if shutil.which("netstat") is not None:
        rc, out = _run_command(["netstat", "-tunlp"])  # 需要 root 才能包含程序名
        if rc == 0:
            results: List[Tuple[int, str]] = []
            seen = set()
            # 示例末列: 1234/nginx
            for line in out.splitlines():
                if f":{port} " not in line and not line.strip().endswith(f":{port}"):
                    continue
                m = re.search(r"\s(\d+)/(\S+)\s*$", line)
                if not m:
                    continue
                pid_i = int(m.group(1))
                name = m.group(2)
                if pid_i in seen:
                    continue
                seen.add(pid_i)
                results.append((pid_i, name))
            if results:
                return results
    return []


def _mac_lookup(port: int) -> List[Tuple[int, str]]:
    # macOS 上 lsof 最可靠
    if shutil.which("lsof") is None:
        return []
    rc, out = _run_command(["lsof", "-nP", f"-i:{port}"])
    if rc != 0:
        return []
    # 列格式: COMMAND PID USER FD TYPE DEVICE SIZE/OFF NODE NAME
    results: List[Tuple[int, str]] = []
    seen = set()
    lines = [l for l in out.splitlines() if l.strip()]
    if not lines:
        return []
    for line in lines[1:]:  # 跳过表头
        parts = re.split(r"\s+", line)
        if len(parts) < 2:
            continue
        name = parts[0]
        try:
            pid = int(parts[1])
        except ValueError:
            continue
        if pid in seen:
            continue
        seen.add(pid)
        results.append((pid, name))
    return results


def _kill_process(pid: int) -> bool:
    """终止指定 PID 的进程（跨平台）
    
    Returns:
        True 如果成功，False 如果失败
    """
    system = platform.system().lower()
    try:
        if system == 'windows':
            # Windows 使用 taskkill
            rc, _ = _run_command(["taskkill", "/F", "/PID", str(pid)])
            return rc == 0
        else:
            # Linux/macOS 使用 kill
            rc, _ = _run_command(["kill", "-9", str(pid)])
            return rc == 0
    except Exception:
        return False


def command_port_who(port: int, kill: bool = False) -> None:
    """打印占用指定端口的进程信息
    
    Args:
        port: 端口号
        kill: 是否终止占用端口的进程
    """
    if not isinstance(port, int) or port <= 0 or port > 65535:
        print("错误: 端口号无效，应为 1-65535 的整数")
        return

    # 1) 尝试 psutil
    results = _try_psutil(port)
    if not results:
        system = platform.system().lower()
        if system == 'windows':
            results = _windows_lookup(port)
        elif system == 'linux':
            results = _linux_lookup(port)
        elif system == 'darwin':
            results = _mac_lookup(port)
        else:
            results = []

    if not results:
        print(f"端口 {port} 未发现被占用或缺少权限/工具无法识别。")
        print("建议：以管理员/root 权限运行，或安装 psutil/lsof。")
        return

    print(f"端口 {port} 被以下进程占用：")
    for pid, name in results:
        print(f"- PID: {pid}\t进程: {name}")

    if kill:
        print("\n正在终止进程...")
        success_count = 0
        for pid, name in results:
            if _kill_process(pid):
                print(f"✓ 已终止进程 PID: {pid} ({name})")
                success_count += 1
            else:
                print(f"✗ 终止进程失败 PID: {pid} ({name})")
        if success_count > 0:
            print(f"\n成功终止 {success_count} 个进程")
        else:
            print("\n未能终止任何进程，可能需要管理员/root 权限")


def command_port_who_by_name(process_name: str, kill: bool = False) -> None:
    """按进程名查询该进程占用的端口（进程名支持不区分大小写、模糊匹配）"""
    if not process_name or not process_name.strip():
        print("错误: 请指定进程名。用法: ypp whoport -n <进程名>")
        return
    process_name = process_name.strip()
    key = process_name.lower()

    all_entries = _get_all_ports()
    # 过滤：进程名包含 key（不区分大小写），按 (port, pid) 去重
    seen: Set[Tuple[int, int]] = set()
    results: List[Tuple[int, int, str]] = []  # (port, pid, name)
    for port, pid, name, _status, _cmdline in all_entries:
        if key not in name.lower():
            continue
        if (port, pid) in seen:
            continue
        seen.add((port, pid))
        results.append((port, pid, name))

    if not results:
        print(f"未发现进程名包含「{process_name}」的端口占用。")
        return

    print(f"进程名包含「{process_name}」的端口占用：")
    for port, pid, name in sorted(results, key=lambda x: (x[1], x[0])):
        print(f"- 端口: {port}\tPID: {pid}\t进程: {name}")

    if kill:
        # 按 PID 去重后终止
        pids_done: Set[int] = set()
        print("\n正在终止进程...")
        success_count = 0
        for _port, pid, name in results:
            if pid in pids_done:
                continue
            pids_done.add(pid)
            if _kill_process(pid):
                print(f"✓ 已终止进程 PID: {pid} ({name})")
                success_count += 1
            else:
                print(f"✗ 终止进程失败 PID: {pid} ({name})")
        if success_count > 0:
            print(f"\n成功终止 {success_count} 个进程")
        else:
            print("\n未能终止任何进程，可能需要管理员/root 权限")




#!/usr/bin/env python3
"""
通用端口映射工具
支持任意IP到IP的端口转发
"""

import os
import sys
import subprocess
import platform
from typing import Optional


def is_windows_system() -> bool:
    """检查是否为 Windows 系统"""
    return platform.system().lower() == "windows"


def is_admin() -> bool:
    """检查是否具有管理员权限"""
    try:
        if is_windows_system():
            import ctypes
            return ctypes.windll.shell32.IsUserAnAdmin()
        else:
            return os.geteuid() == 0
    except:
        return False


def run_netsh_as_admin(command: list) -> bool:
    """以管理员权限执行 netsh 命令"""
    try:
        if not is_windows_system():
            return False
            
        import subprocess
        
        # 使用 PowerShell 以管理员权限执行 netsh 命令
        # 移除第一个 "netsh" 参数，因为 -FilePath 已经指定了程序
        netsh_args = command[1:] if command[0] == "netsh" else command
        # 构建参数列表字符串
        args_str = ",".join([f'"{arg}"' for arg in netsh_args])
        ps_cmd = f'Start-Process -FilePath "netsh" -ArgumentList {args_str} -Verb RunAs -Wait'
        
        result = subprocess.run(['powershell', '-Command', ps_cmd], capture_output=True, text=True, encoding='utf-8', errors='replace')
        
        if result.returncode == 0:
            return True
        else:
            print(f"netsh 命令执行失败: {result.stderr}", file=sys.stderr)
            return False
            
    except Exception as e:
        print(f"以管理员权限执行 netsh 命令失败: {e}", file=sys.stderr)
        return False


def validate_ip_address(ip: str) -> bool:
    """验证IP地址格式"""
    try:
        parts = ip.split('.')
        if len(parts) != 4:
            return False
        for part in parts:
            if not part.isdigit():
                return False
            num = int(part)
            if num < 0 or num > 255:
                return False
        return True
    except:
        return False


def validate_port(port: int) -> bool:
    """验证端口号"""
    return 1 <= port <= 65535


def add_port_mapping(source_ip: str, source_port: int, target_ip: str, target_port: int) -> bool:
    """添加端口映射规则"""
    try:
        # 验证参数
        if not validate_ip_address(source_ip):
            print(f"错误: 源IP地址格式无效: {source_ip}", file=sys.stderr)
            return False
        
        if not validate_ip_address(target_ip):
            print(f"错误: 目标IP地址格式无效: {target_ip}", file=sys.stderr)
            return False
        
        if not validate_port(source_port):
            print(f"错误: 源端口号无效: {source_port}", file=sys.stderr)
            return False
        
        if not validate_port(target_port):
            print(f"错误: 目标端口号无效: {target_port}", file=sys.stderr)
            return False
        
        # 使用 netsh 添加端口映射规则
        cmd = [
            "netsh", "interface", "portproxy", "add", "v4tov4",
            f"listenport={source_port}",
            f"listenaddress={source_ip}",
            f"connectport={target_port}",
            f"connectaddress={target_ip}"
        ]
        
        print(f"添加端口映射: {source_ip}:{source_port} -> {target_ip}:{target_port}")
        
        # 尝试直接执行，如果失败则使用管理员权限
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
            if result.returncode == 0:
                print(f"✓ 端口映射规则添加成功")
                return True
        except subprocess.CalledProcessError:
            # 直接执行失败，尝试使用管理员权限
            print("需要管理员权限，正在请求...")
            if run_netsh_as_admin(cmd):
                print(f"✓ 端口映射规则添加成功")
                return True
            else:
                print(f"✗ 添加端口映射失败", file=sys.stderr)
                return False
        
        return False
    except Exception as e:
        print(f"✗ 添加端口映射时发生错误: {e}", file=sys.stderr)
        return False


def remove_port_mapping(source_ip: str, source_port: int) -> bool:
    """移除端口映射规则"""
    try:
        # 验证参数
        if not validate_ip_address(source_ip):
            print(f"错误: 源IP地址格式无效: {source_ip}", file=sys.stderr)
            return False
        
        if not validate_port(source_port):
            print(f"错误: 源端口号无效: {source_port}", file=sys.stderr)
            return False
        
        cmd = [
            "netsh", "interface", "portproxy", "delete", "v4tov4",
            f"listenport={source_port}",
            f"listenaddress={source_ip}"
        ]
        
        print(f"移除端口映射: {source_ip}:{source_port}")
        
        # 尝试直接执行，如果失败则使用管理员权限
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
            if result.returncode == 0:
                print(f"✓ 端口映射规则移除成功")
                return True
        except subprocess.CalledProcessError:
            # 直接执行失败，尝试使用管理员权限
            print("需要管理员权限，正在请求...")
            if run_netsh_as_admin(cmd):
                print(f"✓ 端口映射规则移除成功")
                return True
            else:
                print(f"✗ 移除端口映射失败", file=sys.stderr)
                return False
        
        return False
    except Exception as e:
        print(f"✗ 移除端口映射时发生错误: {e}", file=sys.stderr)
        return False


def list_port_mappings() -> bool:
    """列出所有端口映射规则"""
    try:
        cmd = ["netsh", "interface", "portproxy", "show", "v4tov4"]
        
        print("当前端口映射规则:")
        print("=" * 80)
        result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
        
        if result.stdout.strip():
            print(result.stdout)
        else:
            print("暂无端口映射规则")
        
        return True
        
    except subprocess.CalledProcessError as e:
        print(f"✗ 获取端口映射规则失败: {e.stderr}", file=sys.stderr)
        return False
    except Exception as e:
        print(f"✗ 获取端口映射规则时发生错误: {e}", file=sys.stderr)
        return False


def command_port_mapping(source_ip: Optional[str] = None, source_port: Optional[int] = None, 
                        target_ip: Optional[str] = None, target_port: Optional[int] = None,
                        action: str = "add") -> None:
    """
    执行端口映射命令
    
    Args:
        source_ip: 源IP地址
        source_port: 源端口
        target_ip: 目标IP地址
        target_port: 目标端口
        action: 操作类型 (add/remove/list)
    """
    # 检查系统
    if not is_windows_system():
        print("错误: 此命令仅在 Windows 系统上支持", file=sys.stderr)
        sys.exit(1)
    
    try:
        if action == "list":
            # 列出所有端口映射规则
            if not list_port_mappings():
                sys.exit(1)
                
        elif action == "add":
            # 添加端口映射规则
            if source_ip is None or source_port is None or target_ip is None or target_port is None:
                print("错误: 添加端口映射需要指定源IP、源端口、目标IP和目标端口", file=sys.stderr)
                print("用法: ypp port <源IP> <源端口> <目标IP> <目标端口>", file=sys.stderr)
                sys.exit(1)
            
            if not add_port_mapping(source_ip, source_port, target_ip, target_port):
                sys.exit(1)
                
        elif action == "remove":
            # 移除端口映射规则
            if source_ip is None or source_port is None:
                print("错误: 移除端口映射需要指定源IP和源端口", file=sys.stderr)
                print("用法: ypp port remove <源IP> <源端口>", file=sys.stderr)
                sys.exit(1)
            
            if not remove_port_mapping(source_ip, source_port):
                sys.exit(1)
        
    except Exception as e:
        print(f"执行端口映射命令时发生错误: {e}", file=sys.stderr)
        sys.exit(1)



#!/usr/bin/env python3
"""
WSL 端口转发工具
"""

import os
import sys
import subprocess
import platform
from typing import Optional


def is_windows_system() -> bool:
    """检查是否为 Windows 系统"""
    return platform.system().lower() == "windows"


def is_admin() -> bool:
    """检查是否具有管理员权限"""
    try:
        if is_windows_system():
            import ctypes
            return ctypes.windll.shell32.IsUserAnAdmin()
        else:
            return os.geteuid() == 0
    except:
        return False


def run_command_as_admin(command: list) -> bool:
    """以管理员权限执行命令"""
    try:
        if not is_windows_system():
            return False
            
        import ctypes
        import subprocess
        
        # 构建完整的命令
        if command[0] == 'netsh':
            # 对于 netsh 命令，直接使用 runas
            full_cmd = ['runas', '/user:Administrator'] + command
        else:
            # 其他命令使用 PowerShell 以管理员权限执行
            ps_cmd = f'Start-Process -FilePath "{command[0]}" -ArgumentList {command[1:]} -Verb RunAs -Wait'
            full_cmd = ['powershell', '-Command', ps_cmd]
        
        # 执行命令
        result = subprocess.run(full_cmd, capture_output=True, text=True, encoding='utf-8', errors='replace')
        
        if result.returncode == 0:
            return True
        else:
            print(f"命令执行失败: {result.stderr}", file=sys.stderr)
            return False
            
    except Exception as e:
        print(f"以管理员权限执行命令失败: {e}", file=sys.stderr)
        return False


def run_netsh_as_admin(command: list) -> bool:
    """以管理员权限执行 netsh 命令"""
    try:
        if not is_windows_system():
            return False
            
        import subprocess
        
        # 使用 PowerShell 以管理员权限执行 netsh 命令
        # 移除第一个 "netsh" 参数，因为 -FilePath 已经指定了程序
        netsh_args = command[1:] if command[0] == "netsh" else command
        # 构建参数列表字符串
        args_str = ",".join([f'"{arg}"' for arg in netsh_args])
        ps_cmd = f'Start-Process -FilePath "netsh" -ArgumentList {args_str} -Verb RunAs -Wait'
        
        result = subprocess.run(['powershell', '-Command', ps_cmd], capture_output=True, text=True, encoding='utf-8', errors='replace')
        
        if result.returncode == 0:
            return True
        else:
            print(f"netsh 命令执行失败: {result.stderr}", file=sys.stderr)
            return False
            
    except Exception as e:
        print(f"以管理员权限执行 netsh 命令失败: {e}", file=sys.stderr)
        return False


def get_wsl_ip() -> Optional[str]:
    """获取 WSL 的 IP 地址"""
    try:
        # 使用 wsl hostname -I 获取 WSL IP
        result = subprocess.run(
            ["wsl", "hostname", "-I"], 
            capture_output=True, 
            text=True, 
            encoding='utf-8',
            check=True
        )
        # 取第一个 IP 地址
        ip = result.stdout.strip().split()[0]
        return ip
    except subprocess.CalledProcessError:
        print("错误: 无法获取 WSL IP 地址", file=sys.stderr)
        return None
    except FileNotFoundError:
        print("错误: 找不到 wsl 命令，请确保已安装 WSL", file=sys.stderr)
        return None


def add_port_forward(host_port: int, wsl_port: int) -> bool:
    """添加端口转发规则"""
    try:
        wsl_ip = get_wsl_ip()
        if not wsl_ip:
            return False
        
        # 使用 netsh 添加端口转发规则
        cmd = [
            "netsh", "interface", "portproxy", "add", "v4tov4",
            f"listenport={host_port}",
            f"listenaddress=0.0.0.0",
            f"connectport={wsl_port}",
            f"connectaddress={wsl_ip}"
        ]
        
        print(f"添加端口转发: 本机 {host_port} -> WSL {wsl_ip}:{wsl_port}")
        
        # 尝试直接执行，如果失败则使用管理员权限
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
            if result.returncode == 0:
                print(f"✓ 端口转发规则添加成功")
                return True
        except subprocess.CalledProcessError:
            # 直接执行失败，尝试使用管理员权限
            print("需要管理员权限，正在请求...")
            if run_netsh_as_admin(cmd):
                print(f"✓ 端口转发规则添加成功")
                return True
            else:
                print(f"✗ 添加端口转发失败", file=sys.stderr)
                return False
        
        return False
    except Exception as e:
        print(f"✗ 添加端口转发时发生错误: {e}", file=sys.stderr)
        return False


def remove_port_forward(host_port: int) -> bool:
    """移除端口转发规则"""
    try:
        cmd = [
            "netsh", "interface", "portproxy", "delete", "v4tov4",
            f"listenport={host_port}",
            f"listenaddress=0.0.0.0"
        ]
        
        print(f"移除端口转发: 本机 {host_port}")
        
        # 尝试直接执行，如果失败则使用管理员权限
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
            if result.returncode == 0:
                print(f"✓ 端口转发规则移除成功")
                return True
        except subprocess.CalledProcessError:
            # 直接执行失败，尝试使用管理员权限
            print("需要管理员权限，正在请求...")
            if run_netsh_as_admin(cmd):
                print(f"✓ 端口转发规则移除成功")
                return True
            else:
                print(f"✗ 移除端口转发失败", file=sys.stderr)
                return False
        
        return False
    except Exception as e:
        print(f"✗ 移除端口转发时发生错误: {e}", file=sys.stderr)
        return False


def list_port_forwards() -> bool:
    """列出所有端口转发规则"""
    try:
        cmd = ["netsh", "interface", "portproxy", "show", "v4tov4"]
        
        print("当前端口转发规则:")
        print("=" * 60)
        result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', check=True)
        
        if result.stdout.strip():
            print(result.stdout)
        else:
            print("暂无端口转发规则")
        
        return True
        
    except subprocess.CalledProcessError as e:
        print(f"✗ 获取端口转发规则失败: {e.stderr}", file=sys.stderr)
        return False
    except Exception as e:
        print(f"✗ 获取端口转发规则时发生错误: {e}", file=sys.stderr)
        return False


def command_wsl_port(host_port: Optional[int] = None, wsl_port: Optional[int] = None, action: str = "add") -> None:
    """
    执行 WSL 端口转发命令
    
    Args:
        host_port: 本机端口
        wsl_port: WSL 端口
        action: 操作类型 (add/remove/list)
    """
    # 检查系统
    if not is_windows_system():
        print("错误: 此命令仅在 Windows 系统上支持", file=sys.stderr)
        sys.exit(1)
    
    try:
        if action == "list":
            # 列出所有端口转发规则
            if not list_port_forwards():
                sys.exit(1)
                
        elif action == "add":
            # 添加端口转发规则
            if host_port is None or wsl_port is None:
                print("错误: 添加端口转发需要指定本机端口和 WSL 端口", file=sys.stderr)
                print("用法: ypp wsl port <本机端口> <WSL端口>", file=sys.stderr)
                sys.exit(1)
            
            if not add_port_forward(host_port, wsl_port):
                sys.exit(1)
                
        elif action == "remove":
            # 移除端口转发规则
            if host_port is None:
                print("错误: 移除端口转发需要指定本机端口", file=sys.stderr)
                print("用法: ypp wsl port remove <本机端口>", file=sys.stderr)
                sys.exit(1)
            
            if not remove_port_forward(host_port):
                sys.exit(1)
        
    except Exception as e:
        print(f"执行端口转发命令时发生错误: {e}", file=sys.stderr)
        sys.exit(1)
